#!/usr/bin/env python
import re
import sys
import getopt
import pickle
import os
import string
import getopt


def lastChar(line):
    i=len(line)-1
    while line[i] in ('\n', '\r', '\t', ' '):
	i=i-1
    return line[i]

def blank(line):
    blank=1
    for i in line:
	if i not in ('\n', '\r', '\t', ' '):
	    blank=0
	    break
    return blank

def comment(line):
    commented=0
    for i in line:
	if i in ('\n', '\r', '\t', ' '):
	    continue
	if i== '#':
	    commented=1
	    break
	else:
	    commented=0
	    break
    return commented

def strip(line):
    for i in range(len(line)):
	if line[i] not in ('\n', '\r', '\t', ' '):
	    return line[i:]
    return line


def processFile(infile, outfile, lineNos):
    counter=0
    lines=infile.readlines()
    lastLineCovered=None

    compBuffer=''

    first=1

    for i in lines:
	counter=counter+1
	if counter in lineNos:
	    outfile.write('> %s' % i)
	    lastLineCovered=i
	elif blank(i) or comment(i):
	    outfile.write('  %s' % i)
	else:
	    outfile.write('! %s' % i)
	    lastLineCovered=None


def getCoverInfo(filename):
    f=open(filename,'r')
    dict=pickle.load(f)

    k=dict.keys()

    fileDict={}
    for i in k:
	if not fileDict.has_key(i[0]):
	    fileDict[i[0]]=[ i[1] ]
	else:
	    fileDict[i[0]].append(i[1])
    
    return fileDict

def exclude(file):
    if file[:len('/usr/local/')]=='/usr/local/':
	return 1
    if file[0]=='<':
	return 1
    if os.path.basename(file)=='pycoverfuncs.py':
	return 1
    return 0

def printUsage():
    print """pyviewc: coverage data viewer

usage:
    pyviewc [options] [files]
    
options:
    -ix, --input=x    which file to read coverage data from
    -h, --help        this screen
    -ere, --exclude=re  exclude files that match the regular expression re
    -Ire, --include=re  include files that match the regular expression re

    * --include's override --excludes
    
    files: which files to output coverage data for
"""
    sys.exit(0)

def getOptions():
    filename=None
    outputs=[]
    excludere=None
    includere=None
    
    try:
	ret=getopt.getopt(sys.argv[1:], "i:e:I:h", ["input=",
                                                    'exclude=',
                                                    'include=',
                                                    'help'])
    except:
	printUsage()
	
    for i in ret[0]:
	if i[0]=='--input' or i[0]=='-i':
	    filename=i[1]
        elif i[0]=='--help' or i[0]=='-h':
            printUsage()
        elif i[0]=='-e' or i[0]=='--exclude':
            excludere = re.compile(i[1])
        elif i[0]=='-I' or i[0]=='--include':
            includere = re.compile(i[1])

    return (filename, ret[1], excludere, includere)

def lookingFor(filename, files):
    if not files: 
	return 1
    for i in files:
	if os.path.basename(i)==os.path.basename(filename):
	    return 1
    return 0

if __name__=='__main__':
    
    (coverFile, files, excludere, includere)=getOptions()

    if coverFile is None:
	try:
	    coverFile=os.environ['PYCOVERFILE']
	except:
	    coverFile='coverage.dat'

    fileDict=getCoverInfo(coverFile)

    for i in fileDict.keys():
	if not lookingFor(i, files):
	    continue
            
	if exclude(i) or (excludere and excludere.search(i)):
            #print "exclude %s?" % i
            if not (includere and includere.search(i)):
                #if includere:
                #    print "match is ", includere.search(i), includere.pattern
                #print "excluding %s" % i
                continue
	try:
	    infile=open(i,'r')
	except:
	    print 'unable to open %s!, skipping...' % i
	    continue

	try:
	    outfileName=i[string.rindex(i,'/')+1:]
	except:
	    outfileName=i
	    print 'using stripped name', i
	outfileName=outfileName+'.covered'
	try:
	    outfile=open(outfileName,'w')
	except:
	    print "couldn't open %s!" % outfileName

	print 'PROCESSING FILE:', i
	processFile(infile, outfile, fileDict[i])

